@@ -53,12 +53,12 @@ the custom TODO words thus defined."
 (defconst org-mem-parser--max-safe-hex-digits
   (- (length (format "%x" most-positive-fixnum)) 1))
 
-(defun org-mem-parser--mk-id (file-name pos)
-  "Reduce FILE-NAME and POS into an `eq'-safe probably-unique fixnum."
-  (+ (string-to-number (substring (secure-hash 'md5 file-name)
-                                  (- org-mem-parser--max-safe-hex-digits))
-                       16)
-     pos))
+(defun org-mem-parser--hash (string)
+  "Summarize STRING as an `eq'-safe hopefully unique fixnum.
+Unlike `sxhash', result is deterministic given the same Emacs executable."
+  (string-to-number (substring (secure-hash 'md5 string)
+                               (- org-mem-parser--max-safe-hex-digits))
+                    16))
 
 (defun org-mem-parser--org-link-display-format (s)
   "Copy of `org-link-display-format'.
@@ -103,6 +103,15 @@ brackets."
 	            nil -1 nil)))
       (and ts (time-convert (encode-time ts) 'integer)))))
 
+
+;;; Links, citations and active timestamps
+
+(defvar org-mem-parser--found-links nil)
+(defvar org-mem-parser--found-active-stamps nil)
+(defconst org-mem-parser--org-ts-regexp
+  "<\\([[:digit:]]\\{4\\}-[[:digit:]]\\{2\\}-[[:digit:]]\\{2\\}\\(?: .*?\\)?\\)>"
+  "Copy of `org-ts-regexp'.")
+
 (defun org-mem-parser--merge-overlapping-regions (regions)
   "Sort and simplify REGIONS, an alist of positions \((BEG . END)).
 When one region overlaps with the next, merge the two."
@@ -120,7 +129,7 @@ When one region overlaps with the next, merge the two."
     (nreverse safe)))
 
 (defun org-mem-parser--scan-visible-text (id-here file internal-entry-id)
-  "Call `org-mem-parser--scan-text-until-1', which see for arguments.
+  "Call `org-mem-parser--scan-text-until', which see for arguments.
 Use the whole visible buffer, but skip regions indicated by
 `org-mem-ignore-regions-regexps'.  Leave point at the end of buffer."
   (goto-char (point-min))
@@ -137,17 +146,12 @@ Use the whole visible buffer, but skip regions indicated by
     (cl-loop
      for (beg . end) in (org-mem-parser--merge-overlapping-regions regions)
      do
-     (org-mem-parser--scan-text-until-1 beg id-here file internal-entry-id)
+     (org-mem-parser--scan-text-until beg id-here file internal-entry-id)
      (goto-char end)))
   (unless (eobp)
-    (org-mem-parser--scan-text-until-1 nil id-here file internal-entry-id)))
+    (org-mem-parser--scan-text-until nil id-here file internal-entry-id)))
 
-(defvar org-mem-parser--found-links nil)
-(defvar org-mem-parser--found-active-stamps nil)
-(defconst org-mem-parser--org-ts-regexp
-  "<\\([[:digit:]]\\{4\\}-[[:digit:]]\\{2\\}-[[:digit:]]\\{2\\}\\(?: .*?\\)?\\)>")
-
-(defun org-mem-parser--scan-text-until-1 (end id-here file internal-entry-id)
+(defun org-mem-parser--scan-text-until (end id-here file internal-entry-id)
   "From here to buffer position END, collect links and active timestamps.
 
 Argument ID-HERE is the ID of the subtree where this function is being
@@ -158,7 +162,7 @@ INTERNAL-ENTRY-ID likewise.
 It is important that END does not extend past any sub-heading, as
 the subheading potentially has an ID of its own."
   (let ((beg (point))
-        LINK-TYPE LINK-PATH LINK-POS LINK-DESC)
+        LINK-TYPE LINK-PATH LINK-POS LINK-DESC SUPPLEMENT)
     ;; Here it may help to know that:
     ;; - `$plain-re' will be morally the same as `org-link-plain-re'
     ;; - `$merged-re' merges the above with `org-link-bracket-re'
@@ -187,9 +191,12 @@ the subheading potentially has an ID of its own."
                 (goto-char (pos-bol))
                 (looking-at-p "[\s\t]*# "))
         ;; Handle a special case opened by Org 9.7 `org-id-link-use-context'
-        (when (and (equal LINK-TYPE "id"))
+        (setq SUPPLEMENT nil)
+        (when (equal LINK-TYPE "id")
           (let ((chop (string-search "::" LINK-PATH)))
-            (when chop (setq LINK-PATH (substring LINK-PATH 0 chop)))))
+            (when chop
+              (setq SUPPLEMENT (substring LINK-PATH (+ 2 chop))
+                    LINK-PATH (substring LINK-PATH 0 chop)))))
         (push (record 'org-mem-link
                       file
                       LINK-POS
@@ -198,6 +205,7 @@ the subheading potentially has an ID of its own."
                       LINK-DESC
                       nil
                       id-here
+                      SUPPLEMENT
                       internal-entry-id)
               org-mem-parser--found-links)
         ;; TODO: Fish any org-ref v3 &citekeys out of LINK-PATH and make a new link
@@ -233,6 +241,7 @@ the subheading potentially has an ID of its own."
                           LINK-DESC
                           t
                           id-here
+                          nil
                           internal-entry-id)
                   org-mem-parser--found-links)))))
 
@@ -243,6 +252,9 @@ the subheading potentially has an ID of its own."
             org-mem-parser--found-active-stamps)))
   (goto-char (or end (point-max))))
 
+
+;;; Properties
+
 (defun org-mem-parser--collect-properties (beg end)
   "Collect Org properties between BEG and END into an alist.
 Assumes BEG and END are buffer positions delimiting a region in
@@ -274,24 +286,34 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
 
 ;;; Main
 
-(defconst org-mem-parser--org-drawer-regexp "^[ \t]*:[_[:word:]-]+:[ \t]*$")
+(defconst org-mem-parser--org-drawer-regexp
+  "^[ \t]*:[_[:word:]-]+:[ \t]*$")
+(defconst org-mem-parser--todo-keyword-re
+  (rx bol (* space) (or "#+todo: " "#+seq_todo: " "#+typ_todo: ")))
+
 (defvar org-mem-parser--outline-regexp nil)
 (defvar org-mem-parser--buf nil)
-(defun org-mem-parser--parse-file (file)
-  "Gather entries, links and other data in FILE."
+
+(defun org-mem-parser--dirty-setup-if-edebug ()
+  "Irreversibly mutate the environment!"
+  ;; In the normal case, we run in subprocesses where this condition fails.
   (when (and (fboundp 'org-mem--mk-work-vars)
              (fboundp 'el-job--ensure-compiled-lib)
              (boundp 'org-mem-load-features)
              (boundp 'org-mem-inject-vars))
-    ;; For debugging in main process; el-job already sets these in subprocesses
-    ;; (the above condition fails if subprocess is calling this).
+    ;; If we are edebugging `org-mem-parser--parse-file' and passing it a FILE
+    ;; directly, these variables have not been set (normally el-job would take
+    ;; care of it, but el-job was never called).  Set them now.
     (dolist (var (org-mem--mk-work-vars))
       (set (car var) (cdr var)))
     (dolist (var org-mem-inject-vars)
       (when (consp var)
         (set (car var) (cdr var))))
     (dolist (lib org-mem-load-features)
-      (load (el-job--ensure-compiled-lib lib))))
+      (load (el-job--ensure-compiled-lib lib)))))
+
+(defun org-mem-parser--init ()
+  "Initialize things, then become a no-op in the normal case."
   (unless (eq org-mem-parser--buf (current-buffer))
     (switch-to-buffer
      (setq org-mem-parser--buf (get-buffer-create " *org-mem-parser*" t))))
@@ -300,13 +322,16 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
           (if $inlinetask-min-level
               (rx-to-string
                `(seq bol (repeat 1 ,(1- $inlinetask-min-level) "*") " "))
-            (rx bol (repeat 1 14 "*") " "))))
+            (rx bol (repeat 1 14 "*") " ")))))
+
+(defun org-mem-parser--parse-file (file)
+  "Gather entries, links and other data in FILE."
+  (org-mem-parser--dirty-setup-if-edebug)
+  (org-mem-parser--init)
   (setq org-mem-parser--found-links nil)
   (setq org-mem-parser--found-active-stamps nil)
   (let ((case-fold-search t)
         (buffer-read-only t)
-        (file-todo-option-re
-         (rx bol (* space) (or "#+todo: " "#+seq_todo: " "#+typ_todo: ")))
         bad-path
         found-entries
         file-data
@@ -367,7 +392,7 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
 
           ;; Scan content before first heading, if any
 
-          (setq INTERNAL-ENTRY-ID (org-mem-parser--mk-id file 0))
+          (setq INTERNAL-ENTRY-ID (org-mem-parser--hash file))
           (while (looking-at-p (rx (*? space) (or "# " "\n")))
             (forward-line))
           (unless (looking-at-p "\\*")
@@ -414,7 +439,7 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
                                          ":" t))))
             (goto-char LEFT)
             (let (collected-todo-lines)
-              (while (re-search-forward file-todo-option-re RIGHT t)
+              (while (re-search-forward org-mem-parser--todo-keyword-re RIGHT t)
                 (push (buffer-substring (point) (pos-eol))
                       collected-todo-lines))
               (when collected-todo-lines
@@ -575,7 +600,7 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
                            (error "Code 9: Couldn't find :END: of drawer"))))
                     nil))
             (setq ID (cdr (assoc "ID" PROPS)))
-            (setq INTERNAL-ENTRY-ID (org-mem-parser--mk-id file HEADING-POS))
+            (setq INTERNAL-ENTRY-ID (+ (org-mem-parser--hash file) HEADING-POS))
             (setq LEFT (point))
             ;; Rough start of body text (just a perf hack, fails gracefully)
             (setq RIGHT (re-search-forward "^[ \t]*[a-bd-z]" nil t))
